function hpaEA(Global)
% <algorithm> <A-G>
% Enhance Selection Pressure for Many-Objective Optimization

    % Randomly initiallize a population
    Population = Global.Initialization();    
    % Generate a set of reference vectors
    [V, ~] = UniformPoint(Global.N, Global.M);
    % Record the nadir point
    maxObj = Inf(1, Global.M);
    % Initialize an array to record the indexes of prominent solutions
    PSI = [];
    
    %% Optimization
    while Global.NotTermination(Population)
        
        % Mating selection
        domNum = size(PSI, 2);
        MatingPool = randi(numel(Population), 1, Global.N-domNum);
        MatingPool = [MatingPool, PSI];
        MatingPool(randperm(size(MatingPool, 2))) = MatingPool(1: end);
        
        Offspring  = Global.Variation(Population(MatingPool));
        
        
        CombinePop = [Population, Offspring];
        % Remove some solutions that cannot dominate the worst point
        fitness = max(CombinePop.objs - repmat(maxObj, numel(CombinePop), 1), [], 2);
        deleteI = fitness > 0;
        CombinePop(deleteI) = [];
        if numel(CombinePop) < 2
            CombinePop = [Population, Offspring];
        end
        % Update the worst point
        tempMax = max(CombinePop.objs);
        replaceI = maxObj > tempMax;
        maxObj(replaceI) = tempMax(replaceI);
        if strcmp(func2str(Global.problem), 'MaF6')
            maxObj = 0.99*maxObj;
            maxObj(maxObj<2) = 2;
        end
        if strcmp(func2str(Global.problem), 'MaF3')
            maxObj = 0.99*maxObj;
            maxObj(maxObj<1) = 1;
        end
        
        
        % Environmental Selection
        [Population, PSI] = hpaEnvironmentalSelection(CombinePop, Global, V);
          
    end
end


